@@ -160,24 +160,10 @@ module type ELIOMREGSIG1 =
        [> `Registrable ]) service
 (** Same as [new_service] followed by [register] *)
 
-    val register_new_service' :
-      ?options:options ->
-      ?sp: Eliom_sessions.server_params ->
-      ?https:bool ->
-      name:string ->
-      get_params:('get, [ `WithoutSuffix ], 'gn) params_type ->
-      ?error_handler:(Eliom_sessions.server_params -> (string * exn) list ->
-                        page Lwt.t) ->
-      (Eliom_sessions.server_params -> 'get -> unit -> page Lwt.t) ->
-      ('get, unit,
-       [> `Nonattached of [> `Get ] na_s ],
-       [ `WithoutSuffix ], 'gn, unit,
-       [> `Registrable ]) service
-(** Same as [new_service'] followed by [register] *)
-
     val register_new_coservice :
       ?options:options ->
       ?sp: Eliom_sessions.server_params ->
+      ?name: string ->
       ?max_use:int ->
       ?timeout:float ->
       ?https:bool ->
@@ -202,6 +188,7 @@ module type ELIOMREGSIG1 =
     val register_new_coservice' :
       ?options:options ->
       ?sp: Eliom_sessions.server_params ->
+      ?name: string ->
       ?max_use:int ->
       ?timeout:float ->
       ?https:bool ->
@@ -221,6 +208,7 @@ module type ELIOMREGSIG1 =
       ?session_name:string ->
       ?secure:bool ->
       sp:Eliom_sessions.server_params ->
+      ?name: string ->
       ?max_use:int ->
       ?timeout:float ->
       ?https:bool ->
@@ -247,6 +235,7 @@ module type ELIOMREGSIG1 =
       ?session_name:string ->
       ?secure:bool ->
       sp:Eliom_sessions.server_params ->
+      ?name: string ->
       ?max_use:int ->
       ?timeout:float ->
       ?https:bool ->
@@ -281,24 +270,10 @@ module type ELIOMREGSIG1 =
         service
 (** Same as [new_post_service] followed by [register] *)
 
-    val register_new_post_service' :
-      ?options:options ->
-      ?sp: Eliom_sessions.server_params ->
-      ?keep_get_na_params:bool ->
-      ?https:bool ->
-      name: string ->
-      post_params:('post, [ `WithoutSuffix ], 'pn) params_type ->
-      ?error_handler:(Eliom_sessions.server_params -> (string * exn) list ->
-                        page Lwt.t) ->
-      (Eliom_sessions.server_params -> unit -> 'post -> page Lwt.t) ->
-      (unit, 'post, [> `Nonattached of [> `Post ] na_s ],
-       [ `WithoutSuffix ], unit, 'pn, [> `Registrable ])
-        service
-(** Same as [new_post_service'] followed by [register] *)
-
     val register_new_post_coservice :
       ?options:options ->
       ?sp: Eliom_sessions.server_params ->
+      ?name: string ->
       ?max_use:int ->
       ?timeout:float ->
       ?https:bool ->
@@ -322,6 +297,7 @@ module type ELIOMREGSIG1 =
     val register_new_post_coservice' :
       ?options:options ->
       ?sp: Eliom_sessions.server_params ->
+      ?name: string ->
       ?max_use:int ->
       ?timeout:float ->
       ?keep_get_na_params:bool ->
@@ -363,6 +339,7 @@ module type ELIOMREGSIG1 =
       ?session_name:string ->
       ?secure:bool ->
       sp:Eliom_sessions.server_params ->
+      ?name: string ->
       ?max_use:int ->
       ?timeout:float ->
       ?https:bool ->
@@ -388,6 +365,7 @@ module type ELIOMREGSIG1 =
       ?session_name:string ->
       ?secure:bool ->
       sp:Eliom_sessions.server_params ->
+      ?name: string ->
       ?max_use:int ->
       ?timeout:float ->
       ?keep_get_na_params:bool ->
@@ -408,6 +386,7 @@ module type ELIOMREGSIG1 =
         ?session_name:string ->
   ?secure:bool ->
         sp:Eliom_sessions.server_params ->
+  ?name: string ->
         ?max_use:int ->
         ?timeout:float ->
       ?https:bool ->
@@ -471,8 +450,8 @@ module MakeRegister = functor
           match get_kind_ service with
           | `Attached attser ->
               let key_kind = get_or_post (get_att_kind_ attser) in
-              let attserget = get_get_state_ attser in
-              let attserpost = get_post_state_ attser in
+              let attserget = get_get_name_ attser in
+              let attserpost = get_post_name_ attser in
               let sgpt = get_get_params_type_ service in
               let sppt = get_post_params_type_ service in
               Eliommod_services.add_service
@@ -481,7 +460,8 @@ module MakeRegister = functor
                 (get_sub_path_ attser)
                 ({Eliom_common.key_state = (attserget, attserpost);
                   Eliom_common.key_kind = key_kind},
-                 ((if attserget = None || attserpost = None
+                 ((if attserget = Eliom_common.Att_no
+                     || attserpost = Eliom_common.Att_no
                  then (anonymise_params_type sgpt,
                        anonymise_params_type sppt)
                  else (0, 0)),
@@ -633,21 +613,10 @@ module MakeRegister = functor
           register ?options ?sp ~service:u ?error_handler page;
           u
 
-        let register_new_service'
-            ?options
-            ?sp
-            ?https
-            ~name
-            ~get_params
-            ?error_handler
-            page =
-          let u = new_service' ?sp ?https ~name ~get_params () in
-          register ?options ?sp ~service:u ?error_handler page;
-          u
-
         let register_new_coservice
             ?options
             ?sp
+            ?name
             ?max_use
             ?timeout
             ?https
@@ -656,7 +625,7 @@ module MakeRegister = functor
             ?error_handler
             page =
           let u = 
-            new_coservice ?max_use ?timeout ?https
+            new_coservice ?name ?max_use ?timeout ?https
               ~fallback ~get_params () 
           in
           register ?options ?sp ~service:u ?error_handler page;
@@ -665,13 +634,16 @@ module MakeRegister = functor
         let register_new_coservice'
             ?options
             ?sp
+            ?name
             ?max_use
             ?timeout
             ?https
             ~get_params
             ?error_handler
             page =
-          let u = new_coservice' ?max_use ?timeout ?https ~get_params () in
+          let u = 
+            new_coservice' ?name ?max_use ?timeout ?https ~get_params () 
+          in
           register ?options ?sp ~service:u ?error_handler page;
           u
 
@@ -680,6 +652,7 @@ module MakeRegister = functor
             ?session_name
             ?secure
             ~sp
+            ?name
             ?max_use
             ?timeout
             ?https
@@ -688,7 +661,7 @@ module MakeRegister = functor
             ?error_handler
             page =
           let u = 
-            new_coservice ?max_use ?timeout ?https
+            new_coservice ?name ?max_use ?timeout ?https
               ~fallback ~get_params () 
           in
           register_for_session
@@ -700,13 +673,14 @@ module MakeRegister = functor
             ?session_name
             ?secure
             ~sp
+            ?name
             ?max_use
             ?timeout
             ?https
             ~get_params
             ?error_handler
             page =
-          let u = new_coservice' ?max_use ?https ~get_params () in
+          let u = new_coservice' ?name ?max_use ?https ~get_params () in
           register_for_session
             ?options ?session_name ?secure ~sp ~service:u ?error_handler page;
           u
@@ -725,28 +699,10 @@ module MakeRegister = functor
           register ?options ?sp ~service:u ?error_handler page_gen;
           u
 
-        let register_new_post_service'
-            ?options
-            ?sp
-            ?keep_get_na_params
-            ?https
-            ~name
-            ~post_params
-            ?error_handler
-            page_gen =
-          let u =
-            new_post_service'
-              ?keep_get_na_params
-              ?https
-              ~name
-              ~post_params:post_params ()
-          in
-          register ?options ?sp ~service:u ?error_handler page_gen;
-          u
-
         let register_new_post_coservice
             ?options
             ?sp
+            ?name
             ?max_use
             ?timeout
             ?https
@@ -755,7 +711,7 @@ module MakeRegister = functor
             ?error_handler
             page_gen =
           let u =
-            new_post_coservice ?max_use ?timeout ?https
+            new_post_coservice ?name ?max_use ?timeout ?https
               ~fallback ~post_params () in
           register ?options ?sp ~service:u ?error_handler page_gen;
           u
@@ -763,6 +719,7 @@ module MakeRegister = functor
         let register_new_post_coservice'
             ?options
             ?sp
+            ?name
             ?max_use
             ?timeout
             ?keep_get_na_params
@@ -772,6 +729,7 @@ module MakeRegister = functor
             page_gen =
           let u =
             new_post_coservice'
+              ?name
               ?keep_get_na_params
               ?max_use
               ?timeout
@@ -785,6 +743,7 @@ module MakeRegister = functor
    let register_new_get_post_coservice'
   ?options
   ?sp
+?name
    ?max_use
    ?timeout
   ?https
@@ -792,7 +751,7 @@ module MakeRegister = functor
    ~post_params
    ?error_handler
    page_gen =
-   let u = new_get_post_coservice'
+   let u = new_get_post_coservice' ?name
    ?max_use ?timeout ?https ~fallback ~post_params () in
    register ?options ?sp ~service:u ?error_handler page_gen;
    u
@@ -803,6 +762,7 @@ module MakeRegister = functor
             ?session_name
             ?secure
             ~sp
+            ?name
             ?max_use
             ?timeout
             ?https
@@ -810,7 +770,7 @@ module MakeRegister = functor
             ~post_params
             ?error_handler
             page_gen =
-          let u = new_post_coservice
+          let u = new_post_coservice ?name
               ?max_use ?timeout ?https ~fallback ~post_params () in
           register_for_session
             ?options ?session_name ?secure ~sp ~service:u ?error_handler page_gen;
@@ -821,6 +781,7 @@ module MakeRegister = functor
             ?session_name
             ?secure
             ~sp
+            ?name
             ?max_use
             ?timeout
             ?keep_get_na_params
@@ -830,6 +791,7 @@ module MakeRegister = functor
             page_gen =
           let u =
             new_post_coservice'
+              ?name
               ?keep_get_na_params
               ?max_use
               ?timeout
@@ -846,6 +808,7 @@ module MakeRegister = functor
   ?session_name
             ?secure
    ~sp
+?name
    ?max_use
    ?timeout
   ?https
@@ -853,7 +816,7 @@ module MakeRegister = functor
    ~post_params
    ?error_handler
    page_gen =
-   let u = new_get_post_coservice'
+   let u = new_get_post_coservice' ?name
    ?max_use ?timeout ?https ~fallback ~post_params () in
    register_for_session
   ?options ?session_name ?secure ~sp ~service:u ?error_handler page_gen;
@@ -911,26 +874,11 @@ module MakeRegister = functor
           ?error_handler:(make_error_handler ?error_handler ())
           (fun sp g p -> page sp g p >>= (fun r -> return (r, [])))
 
-      let register_new_service'
-          ?options
-          ?sp
-          ?https
-          ~name
-          ~get_params
-          ?error_handler
-          page =
-        Cookies.register_new_service'
-          ?options
-          ?sp
-          ?https
-          ~name
-          ~get_params
-          ?error_handler:(make_error_handler ?error_handler ())
-          (fun sp g p -> page sp g p >>= (fun r -> return (r, [])))
 
       let register_new_coservice
           ?options
           ?sp
+          ?name
           ?max_use
           ?timeout
           ?https
@@ -941,6 +889,7 @@ module MakeRegister = functor
         Cookies.register_new_coservice
           ?options
           ?sp
+          ?name
           ?max_use
           ?timeout
           ?https
@@ -952,6 +901,7 @@ module MakeRegister = functor
       let register_new_coservice'
           ?options
           ?sp
+          ?name
           ?max_use
           ?timeout
           ?https
@@ -961,6 +911,7 @@ module MakeRegister = functor
         Cookies.register_new_coservice'
           ?options
           ?sp
+          ?name
           ?max_use
           ?timeout
           ?https
@@ -973,6 +924,7 @@ module MakeRegister = functor
           ?session_name
           ?secure
           ~sp
+          ?name
           ?max_use
           ?timeout
           ?https
@@ -985,6 +937,7 @@ module MakeRegister = functor
         ?session_name
         ?secure
         ~sp
+        ?name
         ?max_use
         ?timeout
         ?https
@@ -998,6 +951,7 @@ module MakeRegister = functor
           ?session_name
           ?secure
           ~sp
+          ?name
           ?max_use
           ?timeout
           ?https
@@ -1009,6 +963,7 @@ module MakeRegister = functor
         ?session_name
         ?secure
         ~sp
+        ?name
         ?max_use
         ?timeout
         ?https
@@ -1033,28 +988,10 @@ module MakeRegister = functor
         ?error_handler:(make_error_handler ?error_handler ())
         (fun sp g p -> page_gen sp g p >>= (fun r -> return (r, [])))
 
-      let register_new_post_service'
-          ?options
-          ?sp
-          ?keep_get_na_params
-          ?https
-          ~name
-          ~post_params
-          ?error_handler
-          page_gen =
-      Cookies.register_new_post_service'
-        ?options
-        ?sp
-        ?keep_get_na_params
-        ?https
-        ~name
-        ~post_params
-        ?error_handler:(make_error_handler ?error_handler ())
-        (fun sp g p -> page_gen sp g p >>= (fun r -> return (r, [])))
-
       let register_new_post_coservice
           ?options
           ?sp
+          ?name
           ?max_use
           ?timeout
           ?https
@@ -1065,6 +1002,7 @@ module MakeRegister = functor
         Cookies.register_new_post_coservice
           ?options
           ?sp
+          ?name
           ?max_use
           ?timeout
           ?https
@@ -1076,6 +1014,7 @@ module MakeRegister = functor
       let register_new_post_coservice'
           ?options
           ?sp
+          ?name
           ?max_use
           ?timeout
           ?keep_get_na_params
@@ -1086,6 +1025,7 @@ module MakeRegister = functor
         Cookies.register_new_post_coservice'
           ?options
           ?sp
+          ?name
           ?max_use
           ?timeout
           ?keep_get_na_params
@@ -1098,6 +1038,7 @@ module MakeRegister = functor
    let register_new_get_post_coservice'
   ?options
   ?sp
+  ?name
           ?max_use
           ?timeout
   ?https
@@ -1108,6 +1049,7 @@ module MakeRegister = functor
    Cookies.register_new_get_post_coservice'
   ?options
    ?sp
+  ?name
           ?max_use
           ?timeout
 ?https
@@ -1123,6 +1065,7 @@ module MakeRegister = functor
           ?session_name
           ?secure
           ~sp
+          ?name
           ?max_use
           ?timeout
           ?https
@@ -1135,6 +1078,7 @@ module MakeRegister = functor
           ?session_name
           ?secure
           ~sp
+          ?name
           ?max_use
           ?timeout
           ?https
@@ -1148,6 +1092,7 @@ module MakeRegister = functor
           ?session_name
           ?secure
           ~sp
+          ?name
           ?max_use
           ?timeout
           ?keep_get_na_params
@@ -1160,6 +1105,7 @@ module MakeRegister = functor
         ?session_name
         ?secure
         ~sp
+        ?name
         ?max_use
         ?timeout
         ?keep_get_na_params
@@ -1174,6 +1120,7 @@ module MakeRegister = functor
   ?session_name
   ?secure
    sp
+  ?name
   ?max_use
   ?timeout
   ?https
@@ -1186,6 +1133,7 @@ module MakeRegister = functor
   ?session_name
   ?secure
   ~sp
+  ?name
   ?max_use
   ?timeout
   ?https
