@@ -47,13 +47,17 @@ let code_of_code_option = function
   | Some c -> c
 
 module Xhtmlreg_(Xhtml_content : Ocsigen_http_frame.HTTP_CONTENT
-                   with type t = [ `Html ] XHTML.M.elt) = struct
+                   with type t = [ `Html ] XHTML.M.elt
+                   and type options = [ `HTML_v03_02 | `HTML_v04_01
+                   | `XHTML_01_00 | `XHTML_01_01 | `Doctype of string ]
+                ) = struct
   open XHTML.M
   open Xhtmltypes
 
   type page = xhtml elt
 
-  type options = unit
+  type options = [ `HTML_v03_02 | `HTML_v04_01
+  | `XHTML_01_00 | `XHTML_01_01 | `Doctype of string ]
 
   module Xhtml_content = struct
 
@@ -86,12 +90,12 @@ module Xhtmlreg_(Xhtml_content : Ocsigen_http_frame.HTTP_CONTENT
 
     let get_etag c = get_etag (add_css c)
 
-    let result_of_content c = result_of_content (add_css c)
+    let result_of_content ?options c = result_of_content ?options (add_css c)
 
   end
 
-  let send ?options ?(cookies=[]) ?charset ?code ~sp content =
-    Xhtml_content.result_of_content content >>= fun r ->
+  let send ?(options = `XHTML_01_01) ?(cookies=[]) ?charset ?code ~sp content =
+    Xhtml_content.result_of_content ~options content >>= fun r ->
     Lwt.return
       (EliomResult
          {r with
@@ -99,8 +103,9 @@ module Xhtmlreg_(Xhtml_content : Ocsigen_http_frame.HTTP_CONTENT
              Eliom_services.cookie_table_of_eliom_cookies ~sp cookies;
             res_code= code_of_code_option code;
             res_charset= (match charset with
-                            | None -> Some (get_config_file_charset sp)
-                            | _ -> charset)
+                            | None -> Some (get_config_default_charset sp)
+                            | _ -> charset
+                         )
          })
 
 end
@@ -309,8 +314,12 @@ module type XHTMLFORMSSIG = sig
 (** Creates the string corresponding to the
     full (absolute) URL of a service applied to its GET parameters.
 
-    Hostname and port are determined from attribute of <host> tag in
-    configuration file, but you can choose the one you want by setting 
+    Default hostname is determined from the [Host] http header of the request
+    (or the attribute of <host> tag in
+    configuration file if the option [<usedefaulthostname/>] is set).
+    Default port is the current port (or another port of the server if
+    you are switching from or to https).
+    But you can choose the hostname or port you want by setting 
     the optional [?hostname] and [?port] parameters here.
  *)
 
@@ -390,13 +399,17 @@ module type XHTMLFORMSSIG = sig
     to make easier the use with reverse proxies.
     But in case of protocol change (if you want to switch to https using
     [~https:true] for example, or if the service imposes https),
-    absolute links will be generated. In that case,
-    hostname and port are determined from attribute of [<host>] tag in
-    configuration file, but you can choose other ones by setting 
+    absolute links will be generated. 
+    In that case,
+    default hostname is determined from the [Host] http header of the request
+    (or the attribute of <host> tag in
+    configuration file if the option [<usedefaulthostname/>] is set).
+    Default port is the current port (or another port of the server if
+    you are switching from or to https).
+    But you can choose the hostname or port you want by setting 
     the optional [?hostname] and [?port] parameters here.
     These options have no effect for relative links.
-    Warning: it may be a security problem to use the hostname sent by the client
-    here.
+
  *)
 
   val css_link : ?a:(link_attrib attrib list) ->
@@ -1537,7 +1550,7 @@ module SubXhtml = functor(T : sig type content end) ->
         let get_etag c = None
 
         let result_of_content c =
-          let x = Xhtmlpretty.xhtml_list_stream c in
+          let x = Xhtmlpretty_streams.xhtml_list_stream c in
           let default_result = default_result () in
           Lwt.return
             {default_result with
@@ -1545,7 +1558,7 @@ module SubXhtml = functor(T : sig type content end) ->
              res_content_type = Some "text/html";
              res_etag = get_etag c;
              res_headers= Http_headers.dyn_headers;
-             res_stream = x
+             res_stream = (x, None)
            }
 
       end
@@ -1566,7 +1579,7 @@ module SubXhtml = functor(T : sig type content end) ->
                 res_cookies= Eliom_services.cookie_table_of_eliom_cookies ~sp cookies;
                 res_code= code_of_code_option code;
                 res_charset= (match charset with
-                | None -> Some (get_config_file_charset sp)
+                | None -> Some (get_config_default_charset sp)
                 | _ -> charset);
               })
 
@@ -1608,7 +1621,7 @@ module Textreg_ = struct
             res_cookies= Eliom_services.cookie_table_of_eliom_cookies ~sp cookies;
             res_code= code_of_code_option code;
             res_charset= (match charset with
-            | None -> Some (get_config_file_charset sp)
+            | None ->  Some (get_config_default_charset sp)
             | _ -> charset);
           })
 
@@ -1635,7 +1648,7 @@ module CssTextreg_ = struct
             res_cookies= Eliom_services.cookie_table_of_eliom_cookies ~sp cookies;
             res_code= code_of_code_option code;
             res_charset= (match charset with
-            | None -> Some (get_config_file_charset sp)
+            | None -> Some (get_config_default_charset sp)
             | _ -> charset);
           })
 
@@ -1663,7 +1676,7 @@ module HtmlTextreg_ = struct
             res_cookies= Eliom_services.cookie_table_of_eliom_cookies ~sp cookies;
             res_code= code_of_code_option code;
             res_charset= (match charset with
-            | None -> Some (get_config_file_charset sp)
+            | None -> Some (get_config_default_charset sp)
             | _ -> charset);
           })
 
@@ -1943,6 +1956,9 @@ end
 
 module String_redirection = MakeRegister(String_redirreg_)
 
+
+
+
 module Redirreg_ = struct
   open XHTML.M
   open Xhtmltypes
@@ -2024,59 +2040,29 @@ module Filesreg_ = struct
   type options = unit
 
   let send ?options ?(cookies=[]) ?charset ?code ~sp filename =
-    let (filename, stat) =
-      (try
-        (* That piece of code has been pasted from staticmod.ml *)
-        let stat = Unix.LargeFile.stat filename in
-        let (filename, stat) =
-          Ocsigen_messages.debug (fun () -> "Eliom.Files - Testing \""^filename^"\".");
-          let path = get_current_sub_path sp in
-          if (stat.Unix.LargeFile.st_kind = Unix.S_DIR)
-          then
-            if (filename.[(String.length filename) - 1]) = '/'
-            then
-              let fn2 = filename^"index.html" in
-              Ocsigen_messages.debug (fun () -> "Eliom.Files - Testing \""^fn2^"\".");
-              (fn2,(Unix.LargeFile.stat fn2))
-            else
-              (if (path= []) || (path = [""])
-              then
-                let fn2 = filename^"/index.html" in
-                Ocsigen_messages.debug (fun () -> "Eliom.Files - Testing \""^fn2^"\".");
-                (fn2,(Unix.LargeFile.stat fn2))
-              else (Ocsigen_messages.debug
-                      (fun () -> "Eliom.Files - "^filename^" is a directory");
-                    raise Ocsigen_Is_a_directory))
-          else (filename, stat)
-        in
-        Ocsigen_messages.debug
-          (fun () ->
-            "Eliom.Files - Looking for \""^filename^"\".");
-
-        if (stat.Unix.LargeFile.st_kind
-              = Unix.S_REG)
-        then begin
-          Unix.access filename [Unix.R_OK];
-          (filename, stat)
-        end
-        else
-          raise (Ocsigen_http_error (Ocsigen_http_frame.Cookies.empty, 404))(* ??? *)
+    let file =
+      try Ocsigen_LocalFiles.resolve (Eliom_sessions.get_request sp) filename
       with
-        (Unix.Unix_error (Unix.EACCES,_,_))
-      | Ocsigen_Is_a_directory
-      | Ocsigen_malformed_url as e -> raise e
-      | e -> raise (Ocsigen_http_error (Ocsigen_http_frame.Cookies.empty, 404)))
+        | Ocsigen_LocalFiles.Failed_403 (* XXXBY : maybe we should signal a true 403? *)
+        | Ocsigen_LocalFiles.Failed_404
+        | Ocsigen_LocalFiles.NotReadableDirectory ->
+            raise Eliom_common.Eliom_404
     in
-    Ocsigen_senders.File_content.result_of_content filename >>= fun r ->
+    Ocsigen_LocalFiles.content ~request:(Eliom_sessions.get_request sp) ~file
+    >>= fun r ->
     Lwt.return
-        (EliomResult
-           {r with
-            res_cookies= Eliom_services.cookie_table_of_eliom_cookies ~sp cookies;
-            res_code= code_of_code_option code;
-            res_charset= (match charset with
-            | None -> Some (get_config_file_charset sp)
-            | _ -> charset);
-          })
+      (EliomResult
+         { r with
+             res_cookies =
+               Eliom_services.cookie_table_of_eliom_cookies ~sp cookies;
+             res_code = code_of_code_option code;
+             res_charset = (match charset with
+                              | None ->
+                                  Some (Ocsigen_charset_mime.find_charset
+                                      filename(get_config_info sp).charset_assoc)
+                              | _ -> charset);
+         })
+
 
 
 end
@@ -2104,7 +2090,7 @@ module Streamlistreg_ = struct
             res_cookies= Eliom_services.cookie_table_of_eliom_cookies ~sp cookies;
             res_code= code_of_code_option code;
             res_charset= (match charset with
-            | None -> Some (get_config_file_charset sp)
+            | None ->  Some (get_config_default_charset sp)
             | _ -> charset);
           })
 
@@ -2112,3 +2098,6 @@ end
 
 module Streamlist = MakeRegister(Streamlistreg_)
 
+
+
+
