@@ -172,7 +172,7 @@ let get_cookie_info
           (catch
              (fun () ->
                Ocsipersist.find
-                 Eliom_common.persistent_cookies_table value >>=
+                 (Lazy.force Eliom_common.persistent_cookies_table) value >>=
                fun (fullsessname, persexp, perstimeout, sessgrp) ->
 
                  Eliommod_sessiongroups.Pers.up value sessgrp >>= fun () ->
@@ -423,7 +423,7 @@ let compute_new_ri_cookies'
   let prefix upo p = match upo with
     | None -> true
     | Some path ->
-        Ocsigen_lib.list_is_prefix
+        Ocsigen_lib.list_is_prefix_skip_end_slash
           (Ocsigen_lib.remove_slash_at_beginning path)
           (Ocsigen_lib.remove_slash_at_beginning p)
   in
