@@ -35,13 +35,6 @@ open Simplexmlparser
 open Ocsigen_http_frame
 
 
-(*****************************************************************************)
-let rec parse_global_config = function
-  | [] -> ()
-  | _ -> badconfig "Unexpected content inside accesscontrol config"
-
-let _ = parse_global_config (Ocsigen_extensions.get_config ())
-
 
 (*****************************************************************************)
 (* Parsing a condition *)
@@ -62,6 +55,35 @@ let rec parse_condition = function
            else
              Ocsigen_messages.debug2 (sprintf "--Access control (ip): %s does not match %s" ri.ri_remote_ip s);
            r)
+    | Element ("ip" as s, _, _) -> badconfig "Bad syntax for tag %s" s
+
+    | Element ("port", ["value", s], []) ->
+        let port =
+          try
+            int_of_string s
+          with Failure _ ->
+            badconfig "Bad port [%s] in <port> condition" s
+        in
+        (fun ri ->
+           let r = ri.ri_server_port = port in
+           if r then
+             Ocsigen_messages.debug2
+               (sprintf "--Access control (port): %d accepted" port)
+           else
+             Ocsigen_messages.debug2
+               (sprintf "--Access control (port): %d not accepted (%d expected)" ri.ri_server_port port);
+           r)
+    | Element ("port" as s, _, _) -> badconfig "Bad syntax for tag %s" s
+
+    | Element ("ssl", [], []) ->
+        (fun ri ->
+           let r = ri.ri_ssl in
+           if r then
+             Ocsigen_messages.debug2 "--Access control (ssl): accepted"
+           else
+             Ocsigen_messages.debug2 "--Access control (ssl): not accepted";
+           r)
+    | Element ("ssl" as s, _, _) -> badconfig "Bad syntax for tag %s" s
 
     | Element ("header", ["name", name; "regexp", reg], []) ->
         let regexp =
@@ -83,6 +105,7 @@ let rec parse_condition = function
            in
            if not r then Ocsigen_messages.debug2 (sprintf "--Access control (header): header %s does not match \"%s\"" name reg);
            r)
+    | Element ("header" as s, _, _) -> badconfig "Bad syntax for tag %s" s
 
     | Element ("method", ["value", s], []) ->
         let meth =
@@ -98,6 +121,7 @@ let rec parse_condition = function
            else Ocsigen_messages.debug
              (fun () -> sprintf "--Access control (method): %s does not match %s" (Framepp.string_of_method ri.ri_method) s);
            r)
+    | Element ("method" as s, _, _) -> badconfig "Bad syntax for tag %s" s
 
     | Element ("protocol", ["value", s], []) ->
         let pr =
@@ -113,6 +137,7 @@ let rec parse_condition = function
            else Ocsigen_messages.debug
              (fun () -> sprintf "--Access control (protocol): %s does not match %s" (Framepp.string_of_proto ri.ri_protocol) s);
            r)
+    | Element ("protocol" as s, _, _) -> badconfig "Bad syntax for tag %s" s
 
     | Element ("path", ["regexp", s], []) ->
         let regexp =
@@ -131,21 +156,22 @@ let rec parse_condition = function
            else Ocsigen_messages.debug
                (fun () -> sprintf "--Access control (path): \"%s\" does not match \"%s\"" ri.ri_sub_path_string s);
            r)
+    | Element ("path" as s, _, _) -> badconfig "Bad syntax for tag %s" s
 
     | Element ("and", [], sub) ->
         let sub = List.map parse_condition sub in
         (fun ri -> List.for_all (fun cond -> cond ri) sub)
+    | Element ("and" as s, _, _) -> badconfig "Bad syntax for tag %s" s
 
     | Element ("or", [], sub) ->
         let sub = List.map parse_condition sub in
         (fun ri -> List.exists (fun cond -> cond ri) sub)
+    | Element ("or" as s, _, _) -> badconfig "Bad syntax for tag %s" s
 
     | Element ("not", [], [sub]) ->
         let sub = parse_condition sub in
         (fun ri -> not (sub ri))
-
-    | Element (("and"|"or"|"not") as t, _, _) ->
-        badconfig "Bad syntax for <%s> condition" t
+    | Element ("not" as s, _, _) -> badconfig "Bad syntax for tag %s" s
 
     | _ ->
         badconfig "Bad syntax for condition"
@@ -154,7 +180,7 @@ let rec parse_condition = function
 (*****************************************************************************)
 (* Parsing filters *)
 
-let parse_config path charset _ parse_fun = function
+let parse_config parse_fun = function
 
   | Element ("if", [], sub) ->
       let (condition, sub) = match sub with
@@ -174,7 +200,7 @@ let parse_config path charset _ parse_fun = function
         | Ocsigen_extensions.Req_found (ri, _)
         | Ocsigen_extensions.Req_not_found (_, ri) ->
             Lwt.return
-              (if condition ri then begin
+              (if condition ri.request_info then begin
                  Ocsigen_messages.debug2 "--Access control: => going into <then> branch";
                  Ocsigen_extensions.Ext_sub_result ithen
                end
@@ -182,16 +208,51 @@ let parse_config path charset _ parse_fun = function
                  Ocsigen_messages.debug2 "--Access control: => going into <else> branch, if any";
                  Ocsigen_extensions.Ext_sub_result ielse
                end))
+  | Element ("if" as s, _, _) -> badconfig "Bad syntax for tag %s" s
+
 
   | Element ("notfound", [], []) ->
       (fun rs ->
          Ocsigen_messages.debug2 "--Access control: taking in charge 404";
-         fail (Ocsigen_http_error (Ocsigen_http_frame.Cookies.empty, 404)))
+         Lwt.return (Ocsigen_extensions.Ext_stop_all
+                       (Ocsigen_http_frame.Cookies.empty, 404)))
+  | Element ("notfound" as s, _, _) -> badconfig "Bad syntax for tag %s" s
+
+  | Element ("nextsite", [], []) ->
+      (function
+         | Ocsigen_extensions.Req_found (_, r) ->
+             Lwt.return (Ocsigen_extensions.Ext_found_stop 
+                           (fun () -> Lwt.return r))
+         | Ocsigen_extensions.Req_not_found (err, ri) ->
+             Lwt.return (Ocsigen_extensions.Ext_stop_site 
+                           (Ocsigen_http_frame.Cookies.empty, 404)))
+
+  | Element ("nexthost", [], []) ->
+      (function
+         | Ocsigen_extensions.Req_found (_, r) ->
+             Lwt.return (Ocsigen_extensions.Ext_found_stop
+                           (fun () -> Lwt.return r))
+         | Ocsigen_extensions.Req_not_found (err, ri) ->
+             Lwt.return (Ocsigen_extensions.Ext_stop_host
+                           (Ocsigen_http_frame.Cookies.empty, 404)))
+  | Element ("nextsite" as s, _, _) -> badconfig "Bad syntax for tag %s" s
+
+  | Element ("stop", [], []) ->
+      (function
+         | Ocsigen_extensions.Req_found (_, r) ->
+             Lwt.return (Ocsigen_extensions.Ext_found_stop
+                           (fun () -> Lwt.return r))
+         | Ocsigen_extensions.Req_not_found (err, ri) ->
+             Lwt.return (Ocsigen_extensions.Ext_stop_all
+                           (Ocsigen_http_frame.Cookies.empty, 404)))
+  | Element ("stop" as s, _, _) -> badconfig "Bad syntax for tag %s" s
 
   | Element ("forbidden", [], []) ->
       (fun rs ->
          Ocsigen_messages.debug2 "--Access control: taking in charge 403";
-         fail (Ocsigen_http_error (Ocsigen_http_frame.Cookies.empty, 403)))
+         Lwt.return (Ocsigen_extensions.Ext_stop_all
+                       (Ocsigen_http_frame.Cookies.empty, 403)))
+  | Element ("forbidden" as s, _, _) -> badconfig "Bad syntax for tag %s" s
 
   | Element ("iffound", [], sub) ->
       let ext = parse_fun sub in
@@ -200,51 +261,40 @@ let parse_config path charset _ parse_fun = function
              Lwt.return (Ext_sub_result ext)
          | Ocsigen_extensions.Req_not_found (err, ri) ->
              Lwt.return (Ocsigen_extensions.Ext_next err))
+  | Element ("iffound" as s, _, _) -> badconfig "Bad syntax for tag %s" s
 
   | Element ("ifnotfound", [], sub) ->
       let ext = parse_fun sub in
       (function
          | Ocsigen_extensions.Req_found (_, r) ->
-             Lwt.return (Ocsigen_extensions.Ext_found r)
+             Lwt.return (Ocsigen_extensions.Ext_found
+                           (fun () -> Lwt.return r))
          | Ocsigen_extensions.Req_not_found (err, ri) ->
              Lwt.return (Ext_sub_result ext))
-
   | Element ("ifnotfound", [("code", s)], sub) ->
       let ext = parse_fun sub in
       let r = Netstring_pcre.regexp ("^"^s^"$") in
       (function
          | Ocsigen_extensions.Req_found (_, r) ->
-             Lwt.return (Ocsigen_extensions.Ext_found r)
+             Lwt.return (Ocsigen_extensions.Ext_found
+                           (fun () -> Lwt.return r))
          | Ocsigen_extensions.Req_not_found (err, ri) ->
              if Netstring_pcre.string_match r (string_of_int err) 0 <> None then
                Lwt.return (Ext_sub_result ext)
              else
                Lwt.return (Ocsigen_extensions.Ext_next err))
+  | Element ("ifnotfound" as s, _, _) -> badconfig "Bad syntax for tag %s" s
 
   | Element (t, _, _) -> raise (Bad_config_tag_for_extension t)
   | _ -> badconfig "(accesscontrol extension) Bad data"
 
 
 
-(*****************************************************************************)
-(** Function to be called at the beginning of the initialisation phase
-    of the server (actually each time the config file is reloaded) *)
-let start_init () =
-  ()
-
-(** Function to be called at the end of the initialisation phase *)
-let end_init () =
-  ()
-
-
-
 
 (*****************************************************************************)
 (** Registration of the extension *)
-let _ = register_extension
-  (fun hostpattern -> parse_config)
-  (fun hostpattern -> parse_config)
-  start_init
-  end_init
-  raise
-
+let () = register_extension
+  ~name:"accesscontrol"
+  ~fun_site:(fun _ _ _ -> parse_config)
+  ~user_fun_site:(fun _ _ _ _ -> parse_config)
+  ()
