@@ -676,11 +676,11 @@ let write_stream_chunked out_ch stream =
 let rec write_stream_raw out_ch stream =
   Ocsigen_stream.next stream >>= fun e ->
   match e with
-    Ocsigen_stream.Finished _ ->
-      Lwt.return ()
-  | Ocsigen_stream.Cont (s, next) ->
-      Lwt_chan.output_string out_ch s >>= fun () ->
-      write_stream_raw out_ch next
+    | Ocsigen_stream.Finished _ ->
+        Lwt.return ()
+    | Ocsigen_stream.Cont (s, next) ->
+        Lwt_chan.output_string out_ch s >>= fun () ->
+        write_stream_raw out_ch next
 
 (*XXX We should check the length of the stream:
   - do not send more than expected
@@ -838,16 +838,16 @@ let send
            Lwt.return ()
          end else begin
            Ocsigen_messages.debug2 "writing body";
-           write_stream ~chunked out_ch res.res_stream
+           write_stream ~chunked out_ch (fst res.res_stream)
          end) >>= fun () ->
          Lwt_chan.flush out_ch (* Vincent: I add this otherwise HEAD answers
                                   are not flushed by the reverse proxy *)
              >>= fun () ->
-         Ocsigen_stream.finalize res.res_stream
+         Ocsigen_stream.finalize (fst res.res_stream)
       )
       (fun e ->
         res.res_stop_stream () >>= fun () ->
-        Ocsigen_stream.finalize res.res_stream >>= fun () ->
+        Ocsigen_stream.finalize (fst res.res_stream) >>= fun () ->
         Lwt.fail e
       )
 
@@ -873,7 +873,8 @@ let send
   in
   let mkcook path exp name c secure =
     Format.sprintf "%s=%s%s%s" name c
-      ("; path=/" ^ Ocsigen_lib.string_of_url_path path)
+(*VVV encode = true? *)
+      ("; path=/" ^ Ocsigen_lib.string_of_url_path ~encode:true path)
       (if secure && slot.sl_ssl then "; secure" else "")^
       (match exp with
       | Some s -> "; expires=" ^
@@ -909,9 +910,14 @@ let send
      match res.res_content_type with
      | None   -> None
      | Some s ->
-         match String.sub s 0 4, res.res_charset with
-         | "text", Some c -> Some (Format.sprintf "%s; charset=%s" s c)
-         | _              -> res.res_content_type)
+         if String.length s >= 4 then
+           match String.sub s 0 4, res.res_charset with
+             | "text", Some "" -> Some s
+             | "text", Some c -> Some (Format.sprintf "%s; charset=%s" s c)
+             | _              -> res.res_content_type
+         else
+           res.res_content_type
+    )
   in
   let mode =
     match mode with
