@@ -71,11 +71,14 @@
 (* constants. Should be configurable *)
 let max_free_open_connections = 2
 
+exception Connection_timed_out
 exception Connection_refused
 exception Pipeline_failed
 
 let (>>=) = Lwt.(>>=)
 
+(* let _ = Ssl_threads.init ()  (* Does not work for now (deadlock) -- 
+                                   bug in ocamlssl *) *)
 let _ = Ssl.init ()
 let sslcontext = ref (Ssl.create_context Ssl.SSLv23 Ssl.Both_context)
 
@@ -263,6 +266,19 @@ let keep_alive_server inet_addr port =
 
 (*****************************************************************************)
 
+let handle_connection_error fd exn = match exn with
+  | Unix.Unix_error (Unix.ECONNREFUSED, _, _) ->
+      Lwt_unix.close fd; Lwt.fail
+        (Ocsigen_http_frame.Http_error.Http_exception
+           (502, Some "Connection refused by distant server", None))
+  | Unix.Unix_error (Unix.ECONNRESET, _, _) ->
+      (* Caused by shutting down the file descriptor after a timeout *)
+      Lwt_unix.close fd; Lwt.fail
+        (Ocsigen_http_frame.Http_error.Http_exception
+           (504, Some "Distant server closed connection", None))
+  | e -> Lwt_unix.close fd; Lwt.fail e
+
+
 let raw_request
     ?client ?(keep_alive = true) ?headers ?(https=false) ?port
     ~content ?content_length ~http_method ~host ~inet_addr ~uri () =
@@ -329,28 +345,33 @@ let raw_request
   in
 
   let new_conn () =
-    let fd = Lwt_unix.socket Unix.PF_INET Unix.SOCK_STREAM 0 in
+    let sockaddr = Unix.ADDR_INET (inet_addr, port) in
+    let fd = 
+      Lwt_unix.socket (Unix.domain_of_sockaddr sockaddr) Unix.SOCK_STREAM 0 
+    in
     Lwt_unix.set_close_on_exec fd;
 
     let thr_conn =
       Lwt.catch
         (fun () ->
-          Lwt_unix.connect
-            fd (Unix.ADDR_INET (inet_addr, port)) >>= fun () ->
+           let timeout =
+             Lwt_timeout.create (Ocsigen_config.get_server_timeout ())
+               (fun () -> Lwt_unix.shutdown fd Unix.SHUTDOWN_RECEIVE);
+           in
+           Lwt_timeout.start timeout;
+           Lwt_unix.connect
+             fd sockaddr >>= fun () ->
 
               (if https then
                 Lwt_ssl.ssl_connect fd !sslcontext
               else
                 Lwt.return (Lwt_ssl.plain fd))
             >>= fun socket ->
-
+            Lwt_timeout.stop timeout;
             Lwt.return (Ocsigen_http_com.create_receiver
                           (Ocsigen_config.get_server_timeout ())
                           Ocsigen_http_com.Answer socket))
-        (function
-          | Unix.Unix_error (Unix.ECONNREFUSED, _, _) ->
-              Lwt_unix.close fd; Lwt.fail Connection_refused
-          | e -> Lwt_unix.close fd; Lwt.fail e)
+        (handle_connection_error fd)
     in
     let gf =
       thr_conn >>= fun conn ->
@@ -438,7 +459,7 @@ let raw_request
     let headers =
       Http_headers.replace
         (Http_headers.name "host")
-        (host^":"^string_of_int port)
+        host
         (match headers with
            | None -> Http_headers.empty
            | Some h -> h)
@@ -665,16 +686,16 @@ let raw_request
 (*****************************************************************************)
 let get ?https ?port ~host ~uri () =
   Ocsigen_lib.get_inet_addr host >>= fun inet_addr ->
-    raw_request
-      ?https
-      ?port
-      ~http_method:Ocsigen_http_frame.Http_header.GET
-      ~content:None
-      ~host
-      ~inet_addr
-      ~uri
-      ()
-      ()
+  raw_request
+    ?https
+    ?port
+    ~http_method:Ocsigen_http_frame.Http_header.GET
+    ~content:None
+    ~host:(match port with None -> host | Some p -> host^":"^string_of_int p)
+    ~inet_addr
+    ~uri
+    ()
+    ()
 
 
 (*VVV missing: post *)
@@ -690,17 +711,20 @@ let basic_raw_request
   | None -> if https then 443 else 80
   | Some p -> p
   in
-  let fd = Lwt_unix.socket Unix.PF_INET Unix.SOCK_STREAM 0 in
+  let sockaddr = Unix.ADDR_INET (inet_addr, port) in
+  let fd = 
+    Lwt_unix.socket (Unix.domain_of_sockaddr sockaddr) Unix.SOCK_STREAM 0 
+  in
   Lwt_unix.set_close_on_exec fd;
 
   Lwt.catch
     (fun () ->
-       Lwt_unix.connect fd (Unix.ADDR_INET (inet_addr, port)) >>= fun () ->
+       Lwt_unix.connect fd sockaddr >>= fun () ->
        (if https then
           Lwt_ssl.ssl_connect fd !sslcontext
         else
           Lwt.return (Lwt_ssl.plain fd)))
-    (fun e -> Lwt_unix.close fd; Lwt.fail e)
+    (handle_connection_error fd)
   >>= fun socket ->
 
   let query = Ocsigen_http_frame.Http_header.Query (http_method, uri) in
@@ -710,7 +734,7 @@ let basic_raw_request
   let headers =
     Http_headers.replace
       (Http_headers.name "host")
-      (host^":"^string_of_int port)
+      host
       (match headers with
          | None -> Http_headers.empty
          | Some h -> h)
